@@ -241,6 +241,33 @@
 "lookup_acs5year_2009"
 
 
+# lookup_acs1year_2018 =========================================================
+#' ACS 1-year 2018 file segment and table lookup data
+#'
+#' There is slightly difference in the lookup tables of each year.
+#'
+#' @docType data
+#'
+#' @format A data.table with 35502 rows and 7 variables
+#' \describe{
+#'   \item{file_segment}{sequence number of segment data files, from "0001" to "0166"}
+#'   \item{table_content}{description of columns in a table}
+#'   \item{reference}{reference of the table content, such as "B01001_002". The reference
+#'         is used to extract data of table content.}
+#'   \item{restriction}{restrictions applied the the table_content}
+#'   \item{table_number}{table number such as "B01001"}
+#'   \item{table_name}{description of table. A table has multiple columns (table_content)}
+#'   \item{universe}{the universe of the data}
+#' }
+#'
+#' @keywords datasets
+#'
+#' @source Check for each year of ACS 1-year and 5-year
+#' \href{https://www.census.gov/programs-surveys/acs/technical-documentation/summary-file-documentation.2015.html}{Sequence Number/Table Number Lookup File}.
+#'
+
+"lookup_acs1year_2018"
+
 
 
 # lookup_acs1year_2017 =========================================================
@@ -890,6 +917,7 @@
 #' \describe{
 #'   \item{table_number}{table number such as "C27013"}
 #'   \item{table_name}{description of the table}
+#'   \item{acs1_2017}{whether the table is available in 2018}
 #'   \item{acs1_2017}{whether the table is available in 2017}
 #'   \item{acs1_2016}{whether the table is available in 2016}
 #'   \item{acs1_2015}{whether the table is available in 2015}
